#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Run tests for drf-cached-instances.

This is for running tests from `setup.py test`.  Most users should use
`manage.py test` instead.
"""

import os
import sys

# Setup path to Django settings
os.environ['DJANGO_SETTINGS_MODULE'] = 'sample_site.settings'

# Add project folder to sys.path
full_path = os.path.abspath(__file__)
my_dir = os.path.dirname(full_path)
proj_dir = os.path.dirname(my_dir)
sys.path.insert(0, proj_dir)

try:
    from django import setup
except ImportError:
    # Django 1.6 and below does not require setup
    setup = lambda: None
else:
    assert setup

from django.test.utils import get_runner
from django.conf import settings


def runtests():
    """Find and run tests for drf-cached-instances."""
    setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    failures = test_runner.run_tests([])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()



"""Tests for drf_cached_instances/mixins.py."""

from datetime import datetime

from django.http import Http404
from django.core.urlresolvers import reverse
from rest_framework.test import APIRequestFactory, APITestCase
from pytz import UTC

from drf_cached_instances.models import CachedModel, CachedQueryset
from sample_poll_app.models import Question
from sample_poll_app.viewsets import QuestionViewSet


class CachedViewMixinTest(APITestCase):

    """Tests for the CachedViewMixin."""

    def test_list_uses_cachedqueryset(self):
        """A GET to the collection (list) uses a CachedQueryset."""
        view = QuestionViewSet()
        view.action = 'list'
        queryset = view.get_queryset()
        self.assertIsInstance(queryset, CachedQueryset)

    def test_update_uses_database(self):
        """A POST to an instance (update) uses the database."""
        view = QuestionViewSet()
        view.action = 'update'
        queryset = view.get_queryset()
        self.assertIsInstance(queryset, type(Question.objects.none()))

    def test_retrieve_object_is_cachedmodel(self):
        """A GET to an instance (retrieve) works with a CachedModel."""
        question = Question.objects.create(
            question_text="What is your quest?",
            pub_date=datetime(2014, 11, 6, 15, 30, 29, 135492, UTC))
        url = reverse('question-detail', kwargs={'pk': question.pk})
        request = APIRequestFactory().get(url)
        view = QuestionViewSet()
        view.action = 'retrieve'
        view.kwargs = {'pk': question.pk}
        view.request = request
        obj = view.get_object()
        self.assertIsInstance(obj, CachedModel)
        self.assertEqual(obj._model, Question)
        expected = {
            'id': question.id,
            'question_text': "What is your quest?",
            'pub_date': question.pub_date,
            'choices': None,
        }
        expected['choices'] = obj._data['choices']
        self.assertEqual(expected, obj._data)

    def test_retrieve_missing_object_is_error(self):
        """A GET to a missing instance is a 404."""
        self.assertFalse(Question.objects.filter(id=666).exists())
        url = reverse('question-detail', kwargs={'pk': 666})
        request = APIRequestFactory().get(url)
        view = QuestionViewSet()
        view.action = 'retrieve'
        view.kwargs = {'pk': 666}
        view.request = request
        self.assertRaises(Http404, view.get_object)

    def test_update_missing_object_is_error(self):
        """A POST to a missing instance is a 404."""
        self.assertFalse(Question.objects.filter(id=666).exists())
        url = reverse('question-detail', kwargs={'pk': 666})
        request = APIRequestFactory().post(
            url, {'question_text': 'What is the capital of Assyria?'})
        view = QuestionViewSet()
        view.action = 'update'
        view.kwargs = {'pk': 666}
        view.request = request
        self.assertRaises(Http404, view.get_object)



"""Tests for drf_cached_instances/cache.py."""

from datetime import datetime, date, timedelta
from json import dumps
import mock

from django.contrib.auth.models import User, Group
from django.test import TestCase
from django.test.utils import override_settings
from pytz import UTC

from drf_cached_instances.cache import BaseCache
from drf_cached_instances.models import PkOnlyModel, PkOnlyQueryset

from sample_poll_app.cache import SampleCache
from sample_poll_app.models import Question, Choice


class SharedCacheTests(object):

    """Define generic cache tests."""

    def test_get_instances_no_specs(self):
        """An empty spec list returns an empty dictionary."""
        instances = self.cache.get_instances([])
        self.assertEqual({}, instances)

    def test_get_instances_cache_miss_no_obj(self):
        """A instance can be loaded without a Django object."""
        date_joined = datetime(2014, 9, 22, 9, 11, tzinfo=UTC)
        user = User.objects.create(
            username='the_user', date_joined=date_joined)
        self.cache.cache and self.cache.cache.clear()
        with self.assertNumQueries(2):
            instances = self.cache.get_instances([('User', user.pk, None)])
        expected = {
            ('User', user.pk): (
                {
                    'id': user.pk,
                    'username': 'the_user',
                    'date_joined': date_joined,
                    'votes': [],
                },
                'drfc_default_User_1',
                user,
            ),
        }
        expected[('User', user.pk)][0]['votes'] = (
            instances[('User', user.pk)][0]['votes'])
        self.assertEqual(expected, instances)

    def test_get_instances_cache_miss_with_obj(self):
        """A instance can be loaded with a Django object."""
        date_joined = datetime(2014, 9, 22, 9, 11, tzinfo=UTC)
        user_pk = User.objects.create(
            username='the_user', date_joined=date_joined).pk
        user = self.cache.user_default_loader(user_pk)
        with self.assertNumQueries(0):
            instances = self.cache.get_instances([('User', user_pk, user)])
        expected = {
            ('User', user_pk): (
                {
                    'id': user_pk,
                    'username': 'the_user',
                    'date_joined': date_joined,
                    'votes': [],
                },
                'drfc_default_User_1',
                user,
            ),
        }
        expected[('User', user_pk)][0]['votes'] = (
            instances[('User', user_pk)][0]['votes'])
        self.assertEqual(expected, instances)

    def test_get_instances_invalid_pk(self):
        """An invalid PK results in an empty instance return."""
        self.assertFalse(User.objects.filter(pk=666).exists())
        instances = self.cache.get_instances([('User', 666, None)])
        self.assertEqual({}, instances)

    def test_update_instance_invalid_model(self):
        """An error is raised updating a model not defined in the Cache."""
        self.assertRaises(AttributeError, self.cache.update_instance, 'Foo', 1)

    def test_update_instance_unhandled_model(self):
        """An error is raised updating a model defined as None in the Cache."""
        instances = self.cache.update_instance('Bar', 666)
        self.assertEqual([], instances)


@override_settings(USE_DRF_INSTANCE_CACHE=True)
class TestCache(SharedCacheTests, TestCase):

    """Test cache functions when the instance cache is enabled."""

    def setUp(self):
        """Setup environment for an enabled cache."""
        self.cache = SampleCache()
        self.cache.cache.clear()
        self.mock_delete = mock.Mock()
        self.cache.cache.delete = self.mock_delete

    def test_cache_is_available(self):
        """When USE_DRF_INSTANCE_CACHE is True, cache is available."""
        self.assertTrue(self.cache.cache)

    def test_get_instance_cache_hit(self):
        """When instance is cached, the database is not queried."""
        key = self.cache.key_for('default', 'User', 123)
        self.assertEqual('drfc_default_User_123', key)
        data = {'id': 123, 'foo': 'bar'}
        self.cache.cache.set(key, dumps(data))
        with self.assertNumQueries(0):
            instances = self.cache.get_instances([('User', 123, None)])
        expected = {('User', 123): (data, key, None)}
        self.assertEqual(expected, instances)

    def test_update_instance_invalidator_only(self):
        """A model can have no serializer but a defined invalidator."""
        user = User.objects.create(username='A user')
        group = Group.objects.create()
        group.user_set.add(user)
        invalid = self.cache.update_instance('Group', group.pk)
        self.assertEqual([('User', user.pk, 'default')], invalid)

    def test_update_instance_deleted_model(self):
        """A deleted instance can still invalidate related instances."""
        self.assertFalse(User.objects.filter(pk=666).exists())
        invalid = self.cache.update_instance('User', 666)
        self.assertEqual([], invalid)
        self.mock_delete.assert_called_once_with('drfc_default_User_666')

    def test_update_instance_cache_string(self):
        """A invalidator can delete cache entries by name."""
        user = User.objects.create(username='username')
        invalid = self.cache.update_instance('User', user.pk)
        self.assertEqual([], invalid)
        self.mock_delete.assert_called_once_with('drfc_user_count')

    def test_update_instance_no_changes(self):
        """When the representation is unchanged, updates do not cascade."""
        user = User.objects.create(
            username='username',
            date_joined=datetime(2014, 11, 5, 22, 2, 16, 735772, UTC))
        key = self.cache.key_for('default', 'User', user.pk)
        representation = {
            'id': user.id,
            'username': 'username',
            'date_joined:DateTime': '1415224936.735772',
            'votes:PKList': {
                'app': 'sample_poll_app',
                'model': 'choice',
                'pks': [],
            },
        }
        self.cache.cache.set(key, dumps(representation))
        self.mock_delete.side_effect = Exception('Not Called')
        invalid = self.cache.update_instance('User', user.pk, user)
        self.assertEqual([], invalid)

    def test_update_instance_with_raw_instance(self):
        """An invalidator loads related PKs with a raw instance."""
        # Create user, then delete auto-created cache
        User.objects.create(username='username')
        self.cache.cache.clear()
        self.mock_delete.reset_mock()
        user = User.objects.get(username='username')

        with self.assertNumQueries(1):
            invalid = self.cache.update_instance('User', user.pk, user)
        self.assertEqual([], invalid)
        self.mock_delete.assert_called_once_with('drfc_user_count')

    def test_update_instance_with_loaded_instance(self):
        """An invalidator skips the database with a loaded instance."""
        # Create user, then delete auto-created cache
        user_pk = User.objects.create(username='username').pk
        user = self.cache.user_default_loader(user_pk)
        self.cache.cache.clear()
        self.mock_delete.reset_mock()

        with self.assertNumQueries(0):
            invalid = self.cache.update_instance('User', user_pk, user)
        self.assertEqual([], invalid)
        self.mock_delete.assert_called_once_with('drfc_user_count')

    def test_delete_called_on_immediate_invalidate(self):
        """An invalidator can ask for immediate invalidation."""
        user = User.objects.create(username='voter')
        question = Question.objects.create(
            question_text='What is your favorite color?',
            pub_date=datetime(2014, 11, 6, 8, 45, 49, 538232, UTC))
        choice = Choice.objects.create(
            question=question, choice_text="Blue. No, Green!")
        choice.voters.add(user)
        invalid = self.cache.choice_default_invalidator(choice)
        expected = [
            ('Question', question.pk, True),
            ('User', user.pk, False),
        ]
        self.assertEqual(expected, invalid)
        to_update = self.cache.update_instance('Choice', choice.pk)
        expected_update = [
            ('Question', question.pk, 'default'),
            ('User', user.pk, 'default'),
        ]
        self.assertEqual(expected_update, to_update)
        self.mock_delete.assert_has_calls([
            mock.call('drfc_user_count'),
            mock.call('drfc_default_Question_%s' % question.pk),
            mock.call('drfc_user_count'),
            mock.call('drfc_default_Question_%s' % question.pk)])
        self.assertEqual(4, self.mock_delete.call_count)

    def test_update_instance_cache_miss_update_only(self):
        """With update_only, cache misses don't update or cascade."""
        user = User.objects.create(username='voter')
        question = Question.objects.create(
            question_text='What is your favorite color?',
            pub_date=datetime(2014, 11, 6, 8, 45, 49, 538232, UTC))
        choice = Choice.objects.create(
            question=question, choice_text="Blue. No, Green!")
        choice.voters.add(user)
        self.cache.cache.clear()
        to_update = self.cache.update_instance(
            'Choice', choice.pk, update_only=True)
        self.assertEqual([], to_update)
        self.mock_delete.assert_has_calls([
            mock.call('drfc_user_count'),
            mock.call('drfc_default_Question_%s' % question.pk),
            mock.call('drfc_user_count')])
        self.assertEqual(3, self.mock_delete.call_count)

    def test_delete_all_versions_one_version(self):
        """Delete all cached instances for a model and ID."""
        self.cache.delete_all_versions("Model", 86)
        self.mock_delete.assert_called_once_with("drfc_default_Model_86")


@override_settings(USE_DRF_INSTANCE_CACHE=True)
class TestVersionsCache(SharedCacheTests, TestCase):

    """Test cache functions when multiple versions are defined."""

    def setUp(self):
        """Setup environment for an enabled cache."""
        self.cache = SampleCache()
        self.cache.versions = ['default', 'v2']
        self.cache.cache.clear()
        self.mock_delete = mock.Mock()
        self.cache.cache.delete = self.mock_delete

    def test_update_instance_unhandled_model(self):
        """An error is raised update a model defined as None in the Cache."""
        self.cache.bar_v2_serializer = None
        self.cache.bar_v2_loader = None
        self.cache.bar_v2_invalidator = None
        super(TestVersionsCache, self).test_update_instance_unhandled_model()

    def test_delete_all_versions_two_versions(self):
        """Delete all cached instances with multiple versions."""
        self.cache.delete_all_versions("Model", 86)
        self.mock_delete.assert_has_calls([
            mock.call("drfc_default_Model_86"),
            mock.call("drfc_v2_Model_86")])


@override_settings(USE_DRF_INSTANCE_CACHE=False)
class TestCacheDisabled(SharedCacheTests, TestCase):

    """Test cache functions when the instance cache is disabled."""

    def setUp(self):
        """Setup environment for a disabled cache."""
        self.cache = SampleCache()

    def test_cache_is_none(self):
        """When USE_DRF_INSTANCE_CACHE is False, cache is None."""
        self.assertIsNone(self.cache.cache)

    def test_invalidate_returns_none(self):
        """When cache is disabled, updating is skipped."""
        with self.assertNumQueries(0):
            invalid = self.cache.update_instance('User', 123)
        self.assertEqual([], invalid)

    def test_delete_all_versions(self):
        """No error when requesting to delete all cached instances."""
        self.cache.delete_all_versions("Model", 86)


class TestFieldConverters(TestCase):

    """Test the built-in field converter methods."""

    def setUp(self):
        """Use a non-customized BaseCache for tests."""
        self.cache = BaseCache()

    def test_date(self):
        """A datetime.date can be stored and retrieved."""
        the_date = date(2014, 9, 22)
        converted = self.cache.field_date_to_json(the_date)
        self.assertEqual(converted, [2014, 9, 22])
        out = self.cache.field_date_from_json(converted)
        self.assertEqual(out, the_date)

    def test_date_string(self):
        """A date string can be stored and retrieved as a datetime.date."""
        the_date = '2015-11-04'
        converted = self.cache.field_date_to_json(the_date)
        self.assertEqual(converted, [2015, 11, 4])
        out = self.cache.field_date_from_json(converted)
        self.assertEqual(out, date(2015, 11, 4))

    def test_date_other_string(self):
        """A non-date string is stored as None."""
        self.assertIsNone(self.cache.field_date_to_json('today'))
        self.assertIsNone(self.cache.field_date_from_json(None))

    def test_datetime_with_ms(self):
        """A datetime with milliseconds can be stored and retrieved."""
        dt = datetime(2014, 9, 22, 8, 52, 0, 123456, UTC)
        converted = self.cache.field_datetime_to_json(dt)
        self.assertEqual(converted, '1411375920.123456')
        out = self.cache.field_datetime_from_json(converted)
        self.assertEqual(out, dt)

    def test_datetime_without_ms(self):
        """A datetime w/o milliseconds can be stored and retrieved."""
        dt = datetime(2014, 9, 22, 8, 52, 0, 0, UTC)
        converted = self.cache.field_datetime_to_json(dt)
        self.assertEqual(converted, 1411375920)
        out = self.cache.field_datetime_from_json(converted)
        self.assertEqual(out, dt)

    def test_datetime_without_timezone(self):
        """A naive datetime is treated as a UTC datetime."""
        dt = datetime(2014, 9, 22, 8, 52, 0, 123456)
        converted = self.cache.field_datetime_to_json(dt)
        self.assertEqual(converted, '1411375920.123456')
        out = self.cache.field_datetime_from_json(converted)
        self.assertEqual(out, datetime(2014, 9, 22, 8, 52, 0, 123456, UTC))

    def test_datetime_string(self):
        """A datetime string can be stored and retrieved as a datetime."""
        dt = "2015-11-04 16:40:04.892566"
        converted = self.cache.field_datetime_to_json(dt)
        self.assertEqual(converted, '1446655204.892566')
        out = self.cache.field_datetime_from_json(converted)
        self.assertEqual(out, datetime(2015, 11, 4, 16, 40, 4, 892566, UTC))

    def test_datetime_other_string(self):
        """A non-datetime string is stored as null."""
        self.assertIsNone(self.cache.field_datetime_to_json("now"))
        self.assertIsNone(self.cache.field_datetime_from_json(None))

    def test_timedelta_without_fractional_seconds(self):
        """Timedelta without fractional seconds can be stored and retrieved."""
        td = timedelta(days=10, hours=1, minutes=9, seconds=5)
        converted = self.cache.field_timedelta_to_json(td)
        self.assertEqual(converted, 868145)
        out = self.cache.field_timedelta_from_json(converted)
        self.assertEqual(out, td)

    def test_timedelta_with_fractional_seconds(self):
        """Timedelta with fractions of a second can be stored and retrieved."""
        td = timedelta(days=10, hours=1, minutes=9, seconds=5, milliseconds=10,
                       microseconds=10)
        converted = self.cache.field_timedelta_to_json(td)
        self.assertEqual(converted, '868145.01001')
        out = self.cache.field_timedelta_from_json(converted)
        self.assertEqual(out, td)

    def test_timedelta_duration_string(self):
        """A duration string can be stored and retrieved as a timedelta."""
        td = "5 3:30:15.99"
        converted = self.cache.field_timedelta_to_json(td)
        self.assertEqual(converted, '444615.99')
        out = self.cache.field_timedelta_from_json(converted)
        self.assertEqual(out, timedelta(5, 12615, 990000))

    def test_timedelta_other_string(self):
        """A non-duration string is stored as null."""
        self.assertIsNone(self.cache.field_timedelta_to_json("a long time"))
        self.assertIsNone(self.cache.field_timedelta_from_json(None))

    def test_pklist(self):
        """A list of primary keys is retrieved as a PkOnlyQueryset."""
        converted = self.cache.field_pklist_to_json(User, (1, 2, 3))
        expected = {
            'app': 'auth',
            'model': 'user',
            'pks': [1, 2, 3],
        }
        self.assertEqual(converted, expected)
        out = self.cache.field_pklist_from_json(converted)
        self.assertIsInstance(out, PkOnlyQueryset)
        self.assertEqual(User, out.model)
        self.assertEqual([1, 2, 3], out.pks)

    def test_pk(self):
        """A primary key is retrieved as a PkOnlyModel."""
        converted = self.cache.field_pk_to_json(User, 1)
        expected = {
            'app': 'auth',
            'model': 'user',
            'pk': 1,
        }
        self.assertEqual(converted, expected)
        out = self.cache.field_pk_from_json(converted)
        self.assertIsInstance(out, PkOnlyModel)
        self.assertEqual(User, out.model)
        self.assertEqual(1, out.pk)



"""Project files for drf-cached-instances."""

# This will make sure the app is always imported when
# Django starts so that shared_task will use this app.
from .celery import app as celery_app
assert celery_app



"""Backports and compatible methods."""

# get_model(app_name, model_name)
# Retrieves Django model class given the app and model name
try:
    # Django 1.8 and later
    from django.apps import apps
    get_model = apps.get_model
except ImportError:  # pragma: nocover
    from django.db.models.loading import get_model
assert get_model

# parse_duration(string)
# Parses a Django or ISO 8601 string into a datetime.timedelta
try:
    # Django 1.8 and later
    from django.utils.dateparse import parse_duration
except:  # pragma: nocover
    from datetime import timedelta
    import re
    from django.utils import six

    standard_duration_re = re.compile(
        r'^'
        r'(?:(?P<days>-?\d+) (days?, )?)?'
        r'((?:(?P<hours>\d+):)(?=\d+:\d+))?'
        r'(?:(?P<minutes>\d+):)?'
        r'(?P<seconds>\d+)'
        r'(?:\.(?P<microseconds>\d{1,6})\d{0,6})?'
        r'$'
    )

    # Support the sections of ISO 8601 date representation that are accepted by
    # timedelta
    iso8601_duration_re = re.compile(
        r'^P'
        r'(?:(?P<days>\d+(.\d+)?)D)?'
        r'(?:T'
        r'(?:(?P<hours>\d+(.\d+)?)H)?'
        r'(?:(?P<minutes>\d+(.\d+)?)M)?'
        r'(?:(?P<seconds>\d+(.\d+)?)S)?'
        r')?'
        r'$'
    )

    def parse_duration(value):
        """Parse a duration string and returns a datetime.timedelta.

        The preferred format for durations in Django is '%d %H:%M:%S.%f'.

        Also supports ISO 8601 representation.
        """
        match = standard_duration_re.match(value)
        if not match:
            match = iso8601_duration_re.match(value)
        if match:
            kw = match.groupdict()
            if kw.get('microseconds'):
                kw['microseconds'] = kw['microseconds'].ljust(6, '0')
            kw = dict([
                (k, float(v)) for k, v in six.iteritems(kw)
                if v is not None])
            return timedelta(**kw)



# -*- coding: utf-8 -*-
"""URL routing patterns for drf-cached-instances."""

from django.conf.urls import patterns, include, url
from django.contrib import admin

from sample_poll_app.urls import urlpatterns as sample_urlpatterns

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
    url(r'', include(sample_urlpatterns)),
)



"""Models for sample poll app.

Based on the "Writing your first Django app" tutorial:
https://docs.djangoproject.com/en/1.7/intro/tutorial01/
"""
from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_delete, post_save, m2m_changed
from django.dispatch import receiver


class Question(models.Model):

    """A poll question."""

    question_text = models.CharField(max_length=200)
    pub_date = models.DateTimeField('date published')


class Choice(models.Model):

    """An answer to a poll question."""

    question = models.ForeignKey(Question, related_name='choices')
    choice_text = models.CharField(max_length=200)
    voters = models.ManyToManyField(User, related_name='votes')


cached_model_names = ('User', 'Question', 'Choice')


@receiver(
    m2m_changed, sender=Choice.voters.through,
    dispatch_uid='m2m_choice_voters_changed_update_cache')
def choice_voters_changed_update_cache(
        sender, instance, action, reverse, model, pk_set, **kwargs):
    """Update cache when choice.voters changes."""
    if action not in ('post_add', 'post_remove', 'post_clear'):
        # post_clear is not handled, because clear is called in
        # django.db.models.fields.related.ReverseManyRelatedObjects.__set__
        # before setting the new order
        return

    if model == User:
        assert type(instance) == Choice
        choices = [instance]
        if pk_set:
            users = list(User.objects.filter(pk__in=pk_set))
        else:
            users = []
    else:
        if pk_set:
            choices = list(Choice.objects.filter(pk__in=pk_set))
        else:
            choices = []
        users = [instance]

    from .tasks import update_cache_for_instance
    for choice in choices:
        update_cache_for_instance('Choice', choice.pk, choice)
    for user in users:
        update_cache_for_instance('User', user.pk, user)


@receiver(post_delete, dispatch_uid='post_delete_update_cache')
def post_delete_update_cache(sender, instance, **kwargs):
    """Update the cache when an instance is deleted."""
    name = sender.__name__
    if name in cached_model_names:
        from .tasks import update_cache_for_instance
        update_cache_for_instance(name, instance.pk, instance)


@receiver(post_save, dispatch_uid='post_save_update_cache')
def post_save_update_cache(sender, instance, created, raw, **kwargs):
    """Update the cache when an instance is created or modified."""
    if raw:
        return
    name = sender.__name__
    if name in cached_model_names:
        delay_cache = getattr(instance, '_delay_cache', False)
        if not delay_cache:
            from .tasks import update_cache_for_instance
            update_cache_for_instance(name, instance.pk, instance)



"""
Django settings for drf-cached-instances project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/

Designed for Heroku and http://12factor.net/config.  Configuration is
overriden by environment variables.

One way to set environment variables for local development in a virtualenv:

$ vi $VIRTUAL_ENV/bin/postactivate
export DJANGO_DEBUG=1
$ vi $VIRTUAL_ENV/bin/predeactivate
unset DJANGO_DEBUG
$ source $VIRTUAL_ENV/bin/postactivate

To set environment variables in heroku environment
$ heroku config
$ heroku config:set DJANGO_DEBUG=1

Environment variables:
ALLOWED_HOSTS - comma-separated list of allowed hosts
DATABASE_URL - See https://github.com/kennethreitz/dj-database-url
DEBUG - 1 to enable, 0 to disable, default disabled
EXTRA_INSTALLED_APPS - comma-separated list of apps to add to INSTALLED_APPS
SECRET_KEY - Overrides SECRET_KEY
SECURE_PROXY_SSL_HEADER - "HTTP_X_FORWARDED_PROTOCOL,https" to enable
STATIC_ROOT - Overrides STATIC_ROOT
"""

# Build paths inside the project like this: rel_path('folder', 'file')
from os import environ, path
import sys
import dj_database_url

BASE_DIR = path.dirname(path.dirname(__file__))


def rel_path(*subpaths):
    """Generate project-relative path."""
    return path.join(BASE_DIR, *subpaths)


# Detect that we're running tests
TESTING = sys.argv[1:2] == ['test']


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
# TODO: Generate a new default secret key for your project
SECRET_KEY = environ.get(
    'SECRET_KEY', 'THIS_IS_A_SUPER_BAD_SECRET_KEY')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = environ.get("DEBUG", '1') in (1, '1')
TEMPLATE_DEBUG = DEBUG

ALLOWED_HOSTS = environ.get('ALLOWED_HOSTS', '').split(',')
if environ.get('SECURE_PROXY_SSL_HEADER'):
    raw = environ['SECURE_PROXY_SSL_HEADER']
    SECURE_PROXY_SSL_HEADER = tuple(raw.split(','))

# Application definition

INSTALLED_APPS = [
    # Django applications
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # 3rd party applications
    'django_extensions',
    'django_nose',
    'rest_framework',

    # Our applications
    'sample_poll_app',
]
if environ.get('EXTRA_INSTALLED_APPS'):
    INSTALLED_APPS + environ['EXTRA_INSTALLED_APPS'].split(',')

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'sample_site.urls'

WSGI_APPLICATION = 'sample_site.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default':
        dj_database_url.config(default='sqlite:///' + rel_path('db.sqlite3'))
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/
if environ.get('STATIC_ROOT'):
    STATIC_ROOT = environ['STATIC_ROOT']
else:
    STATIC_ROOT = 'static'
STATIC_URL = '/static/'


#
# 3rd Party Libraries
#

# Django nose
TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

# Django Debug Toolbar
if DEBUG:
    try:
        import debug_toolbar
    except ImportError:
        pass
    else:
        assert debug_toolbar
        INSTALLED_APPS.append('debug_toolbar')

# Django REST Framework
REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly'
    ]
}
# Celery - async task management
CELERY_TASK_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json']
CELERY_RESULT_SERIALIZER = 'json'
CELERY_ENABLE_UTC = True
CELERY_ALWAYS_EAGER = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True



# -*- coding: utf-8 -*-
# flake8: noqa
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Choice',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('choice_text', models.CharField(max_length=200)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('question_text', models.CharField(max_length=200)),
                ('pub_date', models.DateTimeField(verbose_name=b'date published')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='choice',
            name='question',
            field=models.ForeignKey(to='sample_poll_app.Question'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='choice',
            name='voters',
            field=models.ManyToManyField(related_name='choices', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Packaging setup for drf-cached-instances."""

from drf_cached_instances import __version__ as version

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = open('README.rst').read()
body_tag = ".. Omit badges from docs"
readme_body_start = readme.index(body_tag)
assert readme_body_start
readme_body = readme[readme_body_start + len(body_tag):]
history = open('HISTORY.rst').read().replace('.. :changelog:', '')
long_description = """
==========================================
Cached Instances for Django REST Framework
==========================================
%s

%s
""" % (readme_body, history)

requirements = [
    'Django>=1.6',
    'pytz',
    'djangorestframework',
]

test_requirements = [
    'dj_database_url',
    'django_nose',
    'django_extensions',
    'mock',
    'celery',
]

setup(
    name='drf-cached-instances',
    version=version,
    description='Cached instances for Django REST Framework',
    long_description=long_description,
    author='John Whitlock',
    author_email='john@factorialfive.com',
    url='https://github.com/jwhitlock/drf-cached-instances',
    packages=[
        'drf_cached_instances',
    ],
    package_dir={
        'drf_cached_instances': 'drf_cached_instances',
    },
    include_package_data=True,
    install_requires=requirements,
    license="MPL 2.0",
    zip_safe=False,
    keywords='drf-cached-instances',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='sample_site.runtests.runtests',
    tests_require=test_requirements
)
